//$Id: subtime.cxx,v 2.5 1999/04/08 19:45:04 pinon Exp $
//
// File              : subtime.cxx
//                   :
// Purpose           : define the subtime baseclass.
//                   : UTC, GPST, MET and STST
//                   :
// Member Functions  : get(...) writes the time into the 
//                   :         address of supplied types
//                   : set(...) sets the time 
//                   : print() prints out the time values
//                   :
// Inheritance       : All subtime classes inherit the JD 
//                   :         class and member functions
//                   :
// Example           : GPST gps_time;
//                   : int week;
//                   : double seconds;
//                   :
//                   : gps_time.set(861, 151979.00);
//                   : gps_time.get(&week, &seconds);
//                   : gps_time.print();
//

#include "subtime.h"
#include <stdio.h>



////////////////////////////////////////
//
//  Coordinated Universal Time
//
////////////////////////////////////////

void
UTC::set(int yr, int mo,
	 int dy, int hr,
#ifdef USE_LONG_DOUBLE
	 int mi, long double sc)
#else
	 int mi, double sc)
#endif
{
  year    = yr;   
  month   = mo;    
  day     = dy;
  hours   = hr;
  minutes = mi;
  seconds = sc;

  //sets the value for julian_day
  set_julian_date( yr, mo, dy, hr, mi, sc );

}
  
void
UTC::get(int *yr, int *mo,
	 int *dy, int *hr,
#ifdef USE_LONG_DOUBLE
	 int *mi, long double *sc)
#else
	 int *mi, double *sc)
#endif
{
  *yr = year;   
  *mo = month;
  *dy = day;
  *hr = hours;
  *mi = minutes;
  *sc = seconds;
}

void
UTC::print(void)
{
#ifdef USE_LONG_DOUBLE
  printf("utc time: %d/%d/%d  %0d:%02d:%06.3Lf\n",
#else
  printf("utc time: %d/%d/%d  %0d:%02d:%06.3lf\n",
#endif
	 year,month,day,hours,(unsigned int)minutes,seconds);
}




////////////////////////////////////////
//
//  Global Positioning Satellite Time
//  Time in YYYY/MM/DD HH/MM/SS.SS
//
////////////////////////////////////////

void
GPSclock::set(int yr, int mo,
	 int dy, int hr,
#ifdef USE_LONG_DOUBLE
	 int mi, long double sc)
#else
	 int mi, double sc)
#endif
{
  year    = yr;   
  month   = mo;    
  day     = dy;
  hours   = hr;
  minutes = mi;
  seconds = sc;
  
  //sets the value for julian_day
  set_julian_date( yr, mo, dy, hr, mi, sc );

}
  
void
GPSclock::get(int *yr, int *mo,
	 int *dy, int *hr,
#ifdef USE_LONG_DOUBLE
	 int *mi, long double *sc)
#else
	 int *mi, double *sc)
#endif
{
  *yr = year;   
  *mo = month;
  *dy = day;
  *hr = hours;
  *mi = minutes;
  *sc = seconds;
}

void
GPSclock::print(void)
{
#ifdef USE_LONG_DOUBLE
  printf("gps clock time: %d/%d/%d %0d:%0d:%06.3Lf\n",
#else
  printf("gps clock time: %d/%d/%d %0d:%0d:%06.3lf\n",
#endif
	 year,month,day,hours,minutes,seconds);
}



////////////////////////////////////////
//
//  Global Positioning Satellite Time
//  Time in weeks/sec since GPS epoch
//
////////////////////////////////////////

void
#ifdef USE_LONG_DOUBLE
GPST::set(int wk, long double sc)
#else
GPST::set(int wk, double sc)
#endif
{
  week    = wk;
  seconds = sc;

  julian_day = JULIAN_DAY_REF + week * 7.0 +
			seconds / T_FULL_DAY;
  
}
  
void
#ifdef USE_LONG_DOUBLE
GPST::get(int *wk, long double *sc)
#else
GPST::get(int *wk, double *sc)
#endif
{
  *wk = week;
  *sc = seconds;
}

void
GPST::print(void)
{
#ifdef USE_LONG_DOUBLE
   printf("gps time: %4d %10.3Lf\n",week,seconds);
#else
   printf("gps time: %4d %10.3lf\n",week,seconds);
#endif
//   printf("gps time: %4d %10.3f\n",week,seconds);
}


#ifdef USE_LONG_DOUBLE
long double 
#else
double 
#endif
GPST::operator-(GPST time)
{ 
#ifdef USE_LONG_DOUBLE
  long double result;
#else
  double result;
#endif

  result = (this->week-time.week) * T_FULL_WEEK +
           (this->seconds-time.seconds);

  return result;
}


GPST 
#ifdef USE_LONG_DOUBLE
GPST::operator+(long double time)
#else
GPST::operator+(double time)
#endif
{ 
  GPST result;
  int wk;
  int nwk;
#ifdef USE_LONG_DOUBLE
  long double sc;
#else
  double sc;
#endif
  this->get(&wk,&sc);
  sc = sc + time;
  nwk = (int)(sc/T_FULL_WEEK);
  wk = wk + nwk;
#ifdef USE_LONG_DOUBLE
  sc = sc - (long double)nwk*T_FULL_WEEK;
#else
  sc = sc - (double)nwk*T_FULL_WEEK;
#endif
  result.set(wk,sc);
  return result;
}

GPST 
#ifdef USE_LONG_DOUBLE
GPST::operator-(long double time)
#else
GPST::operator-(double time)
#endif
{ 
  GPST result;
  int wk;
#ifdef USE_LONG_DOUBLE
  long double sc;
  
  wk = int(time/T_FULL_WEEK);
  sc = time - (long double)wk*T_FULL_WEEK;
#else
  double sc;
 
  wk = int(time/T_FULL_WEEK);
  sc = time - (double)wk*T_FULL_WEEK;
#endif

  result.set(this->week-wk,
	     this->seconds-sc);
  return result;
}

int 
GPST::operator==(GPST time)
{
  return (this->week == time.week &&
	  this->seconds == time.seconds);
}

int 
GPST::operator!=(GPST time)
{
  return (this->week != time.week ||
	  this->seconds != time.seconds);
}

int 
GPST::operator<(GPST time)
{
  if (this->week < time.week)
    return TRUE;

  if (this->week == time.week && this->seconds < time.seconds)
    return TRUE;
  else 
    return FALSE;
}


int 
GPST::operator>(GPST time)
{
  if (this->week > time.week)
    return TRUE;

  if (this->week == time.week && this->seconds > time.seconds)
    return TRUE;
  else 
    return FALSE;
}

int 
GPST::operator<=(GPST time)
{
  if (*this == time)
    return TRUE;

  if (*this < time)
    return TRUE;
  else 
    return FALSE;
}

int 
GPST::operator>=(GPST time)
{
  if (*this == time)
    return TRUE;

  if (*this > time)
    return TRUE;
  else 
    return FALSE;
}

////////////////////////////////////////
//
//  Mission Elapsed Time
//
////////////////////////////////////////

void
#ifdef USE_LONG_DOUBLE
MET::set(long double et)
#else
MET::set(double et)
#endif
{
    elapsed_time = et;

    //
    //Convert the et into number of 
    //julian days since launch
    //
    julian_day = elapsed_time/T_FULL_DAY + 0.50;
//    printf("MET JULIAN DAY: %5.5f\n\n", julian_day);

}

void
#ifdef USE_LONG_DOUBLE
MET::get(long double *et)
#else
MET::get(double *et)
#endif
{
    *et = elapsed_time;
}

void
MET::print()
{
#ifdef USE_LONG_DOUBLE
   printf("met time: %10.3Lf\n",elapsed_time);
#else
   printf("met time: %10.3lf\n",elapsed_time);
#endif
}



////////////////////////////////////////
//
//  Shuttle Time Base
//
////////////////////////////////////////

void
#ifdef USE_LONG_DOUBLE
STST::set(int yr, long double sc)
#else
STST::set(int yr, double sc)
#endif
{
    year    = yr;
    seconds = sc;

    //
    //Convert the sc into julian days
    //since launch
    //
    set_julian_date(yr-1,12,31,0,0,sc);

}

void
#ifdef USE_LONG_DOUBLE
STST::get(int *yr, long double *sc)
#else
STST::get(int *yr, double *sc)
#endif
{
    *yr = year;
    *sc = seconds;
}

void
STST::print(void)
{
#ifdef USE_LONG_DOUBLE
   printf("shuttle time: %4d %10.3Lf\n",year,seconds);
#else
   printf("shuttle time: %4d %10.3lf\n",year,seconds);
#endif
//   printf("shuttle time: %4d %10.3f\n",year,seconds);
}


