//
// File              : subTime.h
//                   :
// Purpose           : define the subTime baseclass.
//                   : UTC, GPST, MET and STST
//                   :
// Member Functions  : get(...) writes the time into the 
//                   :         address of supplied types
//                   : set(...) sets the time 
//                   : print() prints out the time values
//                   :
// Inheritance       : All subTime classes inherit the JD 
//                   :         class and member functions
//                   :
// Example           : GPST gps_time;
//                   : int week;
//                   : double seconds;
//                   :
//                   : gps_time.set(861, 151979.00);
//                   : gps_time.get(&week, &seconds);
//                   : gps_time.print();
//
#ifndef subtime_h
#define subtime_h
#include "julian_date.h"


#define DETUT_REF      51.184          // GPS detut at epoch
#define JULIAN_DAY_REF 2444244.500000  // GPS epoch in UTC
#define T_FULL_WEEK    604800.0        // Seconds in a week
#define T_FULL_DAY     86400.0
#define TRUE           1
#define FALSE          0

//--- The following line determines whether long double precision
//      values are used for calculations involving seconds and Julian
//      days.  This is recommended when running on any of the Sparc
//      workstations.  Comment out the line (or delete it) to compile
//      on other systems where long double precision variables are
//      not allowed.

//#define USE_LONG_DOUBLE


////////////////////////////////////////
//
//  Coordinated Universial Time
//
////////////////////////////////////////

class UTC : public JD 
{

private:

  int year;   
  int month;    
  int day;
  int hours;
  int minutes;
#ifdef USE_LONG_DOUBLE
  long double seconds;
#else
  double seconds;
#endif
  
public:

  UTC() {};
  
  UTC(int yr, int mo, int dy, 
#ifdef USE_LONG_DOUBLE
      int hr, int mn, long double sc)
#else
      int hr, int mn, double sc)
#endif
    { 
      set(yr, mo, dy, hr, mn, sc); 
    };

  void set(int yr, int mo,
	   int dy, int hr,
#ifdef USE_LONG_DOUBLE
	   int mn, long double sc);
#else
	   int mn, double sc);
#endif

  void get(int *yr, int *mo,
	   int *dy, int *hr,
#ifdef USE_LONG_DOUBLE
	   int *mn, long double *sc);
#else
	   int *mn, double *sc);
#endif

  void print();
};



////////////////////////////////////////
//
//  Global Positioning Satellite Time
//  Time in weeks/sec since GPS epoch
//
////////////////////////////////////////

class GPST : public JD
{

private:

  int week;  // number of weeks since GPS epoch
#ifdef USE_LONG_DOUBLE
  long double seconds; // number of seconds in week
#else
  double seconds; // number of seconds in week
#endif

public:
  
  GPST() {};
  
#ifdef USE_LONG_DOUBLE
  GPST(int wk, long double sc) {set(wk, sc);};
  void set(int wk, long double sc);
  void get(int *wk, long double *sc);
#else
  GPST(int wk, double sc) {set(wk, sc);};
  void set(int wk, double sc);
  void get(int *wk, double *sc);
#endif

  void print();

#ifdef USE_LONG_DOUBLE
  long double operator-(GPST time);
  GPST operator+(long double time);
  GPST operator-(long double time);
#else
  double operator-(GPST time);
  GPST operator+(double time);
  GPST operator-(double time);
#endif
  int operator==(GPST time);
  int operator!=(GPST time);
  int operator<(GPST time);
  int operator>(GPST time);
  int operator<=(GPST time);
  int operator>=(GPST time);

};



////////////////////////////////////////
//
//  Global Positioning Satellite Time
//  Time in YYYY/MM/DD HH:MM:SS.SS
//
////////////////////////////////////////

class GPSclock : public JD
{

private:

  int year;   
  int month;    
  int day;
  int hours;
  int minutes;
#ifdef USE_LONG_DOUBLE
  long double seconds;
#else
  double seconds;
#endif
  
public:

  GPSclock() {};
  
  GPSclock(int yr, int mo, int dy, 
#ifdef USE_LONG_DOUBLE
	    int hr, int mn, long double sc)
#else
	    int hr, int mn, double sc)
#endif
    { 
      set(yr, mo, dy, hr, mn, sc); 
    };

  void set(int yr, int mo,
	   int dy, int hr,
#ifdef USE_LONG_DOUBLE
	   int mn, long double sc);
#else
	   int mn, double sc);
#endif
  
  void get(int *yr, int *mo,
	   int *dy, int *hr,
#ifdef USE_LONG_DOUBLE
	   int *mn, long double *sc);
#else
	   int *mn, double *sc);
#endif

  void print(void);
};



////////////////////////////////////////
//
//  Mission Elapsed Time
//
////////////////////////////////////////

class MET : public JD
{

private:
  
#ifdef USE_LONG_DOUBLE
  long double elapsed_time; // number of seconds since launch
#else
  double elapsed_time; // number of seconds since launch
#endif

public:
  
  MET () {};
  
#ifdef USE_LONG_DOUBLE
  MET(long double et)
#else
  MET(double et)
#endif
  {
      set(et);
  };

#ifdef USE_LONG_DOUBLE
  void set(long double et);
  void get(long double *et);
#else
  void set(double et);
  void get(double *et);
#endif
  
  void print();
};



////////////////////////////////////////
//
//  Shuttle Time Base
//
////////////////////////////////////////

class STST : public JD
{

private:
  
  int year;       // year of current mission start
#ifdef USE_LONG_DOUBLE
  long double seconds; // seconds of year
#else
  double seconds; // seconds of year
#endif

public:
  
  STST () {};
  
#ifdef USE_LONG_DOUBLE
  STST (int yr, long double sc)
#else
  STST (int yr, double sc)
#endif
  {
    set( yr, sc );
  };

#ifdef USE_LONG_DOUBLE
  void set(int yr, long double sc);
  void get(int *yr, long double *sc);
#else
  void set(int yr, double sc);
  void get(int *yr, double *sc);
#endif

  void print();
};
#endif
